import tornado.ioloop
import tornado.web
import re
from datetime import datetime


def make_app():
    return tornado.web.Application([
        (r"/", MainHandler),
        (r"/api/karma", Api),
    ],
    template_path="templates",
    static_path="static",
    debug=False,
    )


class MainHandler(tornado.web.RequestHandler):
    def get(self):
        self.render("index.html")


karmic_debts = [13, 14, 16, 19]
numerology_chart = {
        'A': 1, 'B': 2, 'C': 3, 'D': 4, 'E': 5, 'F': 6, 'G': 7, 'H': 8, 'I': 9,
        'J': 1, 'K': 2, 'L': 3, 'M': 4, 'N': 5, 'O': 6, 'P': 7, 'Q': 8, 'R': 9,
        'S': 1, 'T': 2, 'U': 3, 'V': 4, 'W': 5, 'X': 6, 'Y': 7, 'Z': 8
    }
numerology_chart_vowels = {'A': 1, 'E': 5, 'I': 9, 'O': 6, 'U': 3}


def reduce_number(num):
    return sum(int(digit) for digit in str(num))


def reduce_number_with_magic(num):
    while num > 9 and num not in [11, 22, 33]:
        num = sum(int(digit) for digit in str(num))
    return num


def reduce_number_with_debt_with_magic(num):
    karmic_debt = None
    if num in karmic_debts:
        karmic_debt = num
    while num > 9 and num not in [11, 22, 33]: 
        num = sum(int(digit) for digit in str(num))
    return karmic_debt
    

def letter_sum(full_name, chart):
    return sum(numerology_chart[letter] for letter in full_name.upper() if letter in chart)


def birth_number(day):
    if day in karmic_debts:
        return True
    return False

    
def life_path(day, month, year):
    reduced_day = reduce_number(day)
    reduced_month = reduce_number(month)
    reduced_year = reduce_number(year)
    total = reduced_day + reduced_month + reduced_year
    if total in karmic_debts:
        return True
    return False


def expression_number(full_name):
    karmic_debt = reduce_number_with_debt_with_magic(letter_sum(full_name, numerology_chart))
    if karmic_debt:
        return True
    return False
    

def soul_urge_number(full_name):
    vowels_total = letter_sum(full_name, numerology_chart_vowels)
    karmic_debt = None
    for num in [vowels_total, reduce_number_with_magic(vowels_total)]:
        if num in karmic_debts:
            karmic_debt = num
            break
    if karmic_debt:
        return True
    return False


def personality_number(full_name):
    consonants_total = letter_sum(full_name, numerology_chart)
    karmic_debt = None
    for num in [consonants_total, reduce_number_with_magic(consonants_total)]:
        if num in karmic_debts:
            karmic_debt = num
            break
    if karmic_debt:
        return True
    return False


def calculate_minor_expression_number(first_name):
    karmic_debt = reduce_number_with_debt_with_magic(letter_sum(first_name, numerology_chart))
    if karmic_debt:
        return True
    return False


def calculate_personal_year_number(day, month):
    total = month + day + datetime.now().year
    karmic_debt = reduce_number_with_debt_with_magic(total)
    if karmic_debt:
        return True
    return False


def calculate_month_day_combination(day, month):
    total = day + month
    karmic_debt = reduce_number_with_debt_with_magic(total)
    if karmic_debt:
        return True
    return False


class Api(tornado.web.RequestHandler):
    def post(self):
        self.set_header('Content-Type', 'application/json')
        day = self.get_argument("day")
        if day.isdigit() and 1 <= int(day) <= 31:
            day = day.zfill(2)
        else:
            self.write({"data":100500,"error":"Invalid day format"})
            return
        day = int(day)

        month = self.get_argument("month")
        if month.isdigit() and 1 <= int(month) <= 12:
            month = month.zfill(2)
        else:
            self.write({"data":100500,"error":"Invalid month format"})
            return
        month = int(month)

        year = self.get_argument("year")
        if not (year.isdigit() and len(year) == 4):
            self.write({"data":100500,"error":"Invalid year format"})
            return
        year=int(year)
        
        firstname = self.get_argument("firstname")
        lastname = self.get_argument("lastname")
        if not (re.match("^[a-zA-Z]+$", firstname) and re.match("^[a-zA-Z]+$", lastname)):
            self.write({"data":100500,"error":"First and last names must contain only letters"})
            return

        full_name = firstname + lastname
        full_name = full_name.replace(" ", "").upper()

        results = [birth_number(day), life_path(day, month, year), expression_number(full_name), soul_urge_number(full_name), personality_number(full_name), calculate_minor_expression_number(firstname), calculate_personal_year_number(day,month), calculate_month_day_combination(day, month)]
        true_count = sum(1 for condition in results if condition)
        true_count += 1 # +1 sin for using internet 
        self.write({"data":str(true_count)})


if __name__ == "__main__":
    app = make_app()
    app.listen(8000)
    tornado.ioloop.IOLoop.current().start()