import tornado.web
from models import User, Session
import pyotp
import pyotp
from utils import validate_input, url_to_qrcode_data_uri

class TwoFAEnablerHandler(tornado.web.RequestHandler):
    async def post(self):
        user_id = self.get_secure_cookie("user")
        current_role = self.get_secure_cookie("role")
        if not user_id or not current_role:
            self.redirect("/login")
            return
        
        session = Session()
        user = session.query(User).filter(User.id == int(user_id)).first()

        if user:
            if not user.totp_enabled:
                self.redirect("/2fa_setup")
            else:
                user.totp_secret = None
                user.totp_enabled = False
                session.commit()

        session.close()
        self.redirect("/account")


class TwoFASecretHandler(tornado.web.RequestHandler):
    async def get(self):
        user_id = self.get_secure_cookie("user")
        current_role = self.get_secure_cookie("role")
        if not user_id or not current_role:
            self.redirect("/login")
            return
        session = Session()
        user = session.query(User).filter(User.id == int(user_id)).first()
        if not user.totp_enabled:
            user.totp_secret = pyotp.random_base32()
            session.commit()
            totp_uri = pyotp.totp.TOTP(user.totp_secret).provisioning_uri(user.username, issuer_name="Indish")
            data_uri = await url_to_qrcode_data_uri(totp_uri)
            self.render("2fa_secret.html", totp_uri=data_uri, username=user.username)
        else:
            self.render("error.html", type="error", message="2FA already configured")


    async def post(self):
        user_id = self.get_secure_cookie("user")
        current_role = self.get_secure_cookie("role")
        if not user_id or not current_role:
            self.redirect("/login")
            return
        
        otp_code = await validate_input(self.get_arguments("otp"))
        if not otp_code:
            self.render("error.html", type="error", message="OTP was not provided")

        session = Session()
        user = session.query(User).filter(User.id == int(user_id)).first()

        if user and user.totp_secret:
            totp = pyotp.TOTP(user.totp_secret)
            if totp.verify(otp_code):
                user.totp_enabled = True
                session.commit()
                self.redirect("/account")
            else:
                self.render("error.html", type="error", message="Invalid OTP")
        else:
            self.render("error.html", type="error", message="2FA is not configured")
        session.close()


class TwoFAHandler(tornado.web.RequestHandler):
    async def get(self):
        user_id = self.get_secure_cookie("tmp_user")
        if not user_id:
            self.redirect("/login")
            return

        session = Session()
        user = session.query(User).filter(User.id == int(user_id)).first()

        if not user:
            self.clear_cookie("user")
            self.redirect("/login")
            return

        self.render("2fa.html", otp_secret=user.totp_secret if user.totp_secret else None, username=user_id)
        session.close()


    async def post(self):
        user_id = self.get_secure_cookie("tmp_user")
        otp_code = await validate_input(self.get_arguments("otp"))
        if not otp_code:
            self.render("error.html", type="error", message="OTP was not provided")

        if not user_id:
            self.redirect("/login")
            return

        user = Session().query(User).filter(User.id == int(user_id)).first()
        if user and user.totp_secret:
            totp = pyotp.TOTP(user.totp_secret)
            if totp.verify(otp_code):
                self.clear_cookie("tmp_user")
                self.set_secure_cookie("user", str(user_id.decode("utf-8")))
                self.redirect("/role")
            else:
                self.render("error.html", type="error", message="Invalid OTP")
        else:
            self.render("error.html", type="error", message="2FA is not configured")
        return
