import qrcode
import io
import base64
import re
import aiohttp


async def validate_input(input, var_type=None, var_range=None):
    if not input:
        return False
    if isinstance(input, list):
        if len(input) == 6:
            result = ''.join(input)
        else:
            result = input[0]
        if result.isdigit():
            result = int(result)
        else:
            if not re.match("^[a-zA-Z]+$", result):
                return False
        if var_type:
            if not isinstance(result, var_type):
                return False
        if var_range:
            start_val, end_val = map(int, var_range.split('-'))
            if not result in range(start_val, end_val + 1):
                return False
        return result
    return False
        

async def url_to_qrcode_data_uri(url):
    qr = qrcode.QRCode(
        version=1,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=10,
        border=4,
    )
    qr.add_data(url)
    qr.make(fit=True)
    img = qr.make_image(fill_color="black", back_color="white")
    buffered = io.BytesIO()
    img.save(buffered, format="PNG")
    img_str = base64.b64encode(buffered.getvalue()).decode()
    data_uri = f"data:image/png;base64,{img_str}"
    return data_uri


async def set_karma(day, month, year, firstname, lastname):
    url = "http://indish_karma:8000/api/karma"
    data = {"day": day, "month": month, "year": year, "firstname": firstname, "lastname": lastname}
    async with aiohttp.ClientSession() as session:
        async with session.post(url, data=data) as response:
            if response.status == 200:
                data = await response.json()
            else:
                return None
    return data.get("data")
